/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.git.data;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Represents some file contained in the Git repository and mainly provides the methods to deal with the conflicts in this file.
 *
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 */
public class File {

    private final java.io.File file;

    private List<String> lines = new ArrayList<String>();

    public File(java.io.File file) throws IOException {
        super();
        this.file = file;
        readFile();
    }

    private void readFile() throws IOException {
        FileInputStream is = new FileInputStream(file);
        InputStreamReader isr = new InputStreamReader(is, "ASCII");
        BufferedReader br = new BufferedReader(isr);
        try {
            String line;
            while ((line = br.readLine()) != null) {
                lines.add(line);
            }
        } finally {
            br.close();
            isr.close();
            is.close();
        }
        findConflicts();
    }

    protected List<Conflict> findConflicts() {
        List<Conflict> retval = new ArrayList<Conflict>();
        int pos = 0;
        Conflict conflict;
        while ((conflict = Conflict.parseConflict(lines, pos)) != null) {
            retval.add(conflict);
            pos = conflict.getLastLine() + 1;
        }
        return retval;
    }

    public Iterator<Conflict> conflictIterator() {
        List<Conflict> conflicts = findConflicts();
        return conflicts.iterator();
    }

    public void solve(Conflict conflict, List<String> replacement) {
        removeLines(conflict.getFirstLine(), conflict.getLastLine());
        insertLines(conflict.getFirstLine(), replacement);
        findConflicts();
    }

    private void removeLines(int start, int end) {
        for (int amount = end - start + 1; amount > 0; amount--) {
            lines.remove(start);
        }
    }

    private void insertLines(int pos, List<String> toInsert) {
        int myPos = pos;
        for (String line : toInsert) {
            lines.add(myPos++, line);
        }
    }

    public void save() throws IOException {
        FileOutputStream fos = new FileOutputStream(file);
        OutputStreamWriter osw = new OutputStreamWriter(fos, "ASCII");
        BufferedWriter bw = new BufferedWriter(osw);
        try {
            bw.write(toString());
        } finally {
            bw.close();
            osw.close();
            fos.close();
        }
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (String line : lines) {
            sb.append(line);
            sb.append('\n');
        }
        return sb.toString();
    }
}
